@@ -23,6 +23,11 @@ if typing.TYPE_CHECKING:
 	from .main import App
 
 
+UrwidMarkup: 'typing.TypeAlias' = 'str|tuple[str, object]|list[object]'
+LogLines: 'typing.TypeAlias' = 'list[tuple[str, str|None, str]]'
+LinesOfUrwidMarkup: 'typing.TypeAlias' = 'list[UrwidMarkup]'
+
+
 ALIGN_CHAR = "^"
 
 #https://matthew-brett.github.io/curious-git/git_object_types.html
@@ -56,7 +61,7 @@ hint_timeout = _("Either speed up the command (e.g. in case of a grep ignoring u
 class CommandFunction:
 
 	def __init__(self,
-		method: 'collections.abc.Callable[[LogModel, collections.abc.Sequence[str]], list[tuple[str, str, str]]] | collections.abc.Callable[[DetailsModel, collections.abc.Sequence[str]], list[str|list[object]|tuple[str, object]]]',
+		method: 'collections.abc.Callable[[LogModel, collections.abc.Sequence[str]], LogLines] | collections.abc.Callable[[DetailsModel, collections.abc.Sequence[str]], LinesOfUrwidMarkup]',
 		arguments: str,
 		description: str,
 		settings: 'collections.abc.Sequence[str]',
@@ -68,11 +73,11 @@ class CommandFunction:
 		functools.update_wrapper(self, method)
 
 	@typing.overload
-	def __call__(self, model: 'LogModel', args: 'collections.abc.Sequence[str]') -> 'list[tuple[str, str, str]]':
+	def __call__(self, model: 'LogModel', args: 'collections.abc.Sequence[str]') -> 'LogLines':
 		pass
 
 	@typing.overload
-	def __call__(self, model: 'DetailsModel', args: 'collections.abc.Sequence[str]') -> 'list[str|list[object]|tuple[str, object]]':
+	def __call__(self, model: 'DetailsModel', args: 'collections.abc.Sequence[str]') -> 'LinesOfUrwidMarkup':
 		pass
 
 	def __call__(self, model: 'typing.Any', args: 'collections.abc.Sequence[str]') -> 'typing.Any':
@@ -83,9 +88,9 @@ def command_function(
 	arguments: str,
 	description: str,
 	settings: 'collections.abc.Sequence[str]',
-) -> 'collections.abc.Callable[[collections.abc.Callable[[LogModel, collections.abc.Sequence[str]], list[tuple[str, str, str]]] | collections.abc.Callable[[DetailsModel, collections.abc.Sequence[str]], list[str|list[object]|tuple[str, object]]]], CommandFunction]':
+) -> 'collections.abc.Callable[[collections.abc.Callable[[LogModel, collections.abc.Sequence[str]], LogLines] | collections.abc.Callable[[DetailsModel, collections.abc.Sequence[str]], LinesOfUrwidMarkup]], CommandFunction]':
 
-	def decorator(method: 'collections.abc.Callable[[LogModel, collections.abc.Sequence[str]], list[tuple[str, str, str]]] | collections.abc.Callable[[DetailsModel, collections.abc.Sequence[str]], list[str|list[object]|tuple[str, object]]]') -> CommandFunction:
+	def decorator(method: 'collections.abc.Callable[[LogModel, collections.abc.Sequence[str]], LogLines] | collections.abc.Callable[[DetailsModel, collections.abc.Sequence[str]], LinesOfUrwidMarkup]') -> CommandFunction:
 		return CommandFunction(method, arguments=arguments, description=description, settings=settings)
 
 	return decorator
@@ -420,7 +425,7 @@ class LogModel(Model):
 	reo_hash_id = re.compile(r"(?:\b|(?<=m))(%(re_hex)s{7,40})\b" % dict(re_hex=re_hex), re.IGNORECASE)
 	reo_stash_id = re.compile(r"stash@\{\d+\}", re.IGNORECASE)
 
-	def get_lines(self):
+	def get_lines(self) -> 'LogLines':
 		"""return a list of 3-tuples (line, id, type), each representing one line"""
 		# committed changes
 		self.cmd = self.cmd_log
@@ -435,7 +440,7 @@ class LogModel(Model):
 
 		return log
 
-	def format_command_error(self, e, setting):
+	def format_command_error(self, e: CommandError, setting: str) -> 'LogLines':
 		if isinstance(e.exception, TimeoutExpired):
 			hint = self.ln_pattern_error_hint.format(hint=hint_timeout)
 		else:
@@ -460,7 +465,7 @@ class LogModel(Model):
 			settings.COLOR_LOG_STAGED,
 		],
 	)
-	def status(self, args: 'collections.abc.Sequence[str]') -> 'list[tuple[str, str, str]]':
+	def status(self, args: 'collections.abc.Sequence[str]') -> 'LogLines':
 		# uncommitted changes
 		if args:
 			cmd = ['$status'] + list(args)
@@ -486,7 +491,7 @@ class LogModel(Model):
 		if not self.show_untracked:
 			untracked = False
 
-		log: 'list[tuple[str, str, str]]' = []
+		log: 'LogLines' = []
 		if staged:
 			ln = self.format_log_entry(settings.COLOR_LOG_STAGED, title_staged)
 			log.insert(0, (ln, ID_STAGED, TYPE_OTHER))
@@ -520,12 +525,12 @@ class LogModel(Model):
 			settings.COLOR_LOG_STASHES,
 		],
 	)
-	def stashes(self, args: 'collections.abc.Sequence[str]') -> 'list[tuple[str, str, str]]':
+	def stashes(self, args: 'collections.abc.Sequence[str]') -> 'LogLines':
 		if args:
 			cmd = ['$stashes'] + list(args)
 			raise CommandError(cmd, "stashes takes no arguments")
 
-		log: 'list[tuple[str, str, str]]' = []
+		log: 'LogLines' = []
 
 		if self.show_stashed_changes:
 			self.cmd = self.cmd_stash_list
@@ -558,11 +563,12 @@ class LogModel(Model):
 			settings.COLOR_LOG_TODO,
 		],
 	)
-	def todo(self, args: 'collections.abc.Sequence[str]') -> 'list[tuple[str, str, str]]':
+	def todo(self, args: 'collections.abc.Sequence[str]') -> 'LogLines':
 		if args:
 			cmd = ['$todo'] + list(args)
 			raise CommandError(cmd, "todo takes no arguments")
 
+		out_true: 'LogLines'
 		out_true = [(self.format_log_entry(settings.COLOR_LOG_TODO, title_todo_log), ID_TODO, TYPE_TODO)]
 		if self.show_list_of_todo_flags:
 			return out_true
@@ -628,7 +634,7 @@ class LogModel(Model):
 
 class StashesModel(LogModel):
 
-	def get_lines(self):
+	def get_lines(self) -> 'LogLines':
 		self.cmd = self.cmd_stash_list
 		try:
 			stashed = Model.get_lines(self)
@@ -643,7 +649,7 @@ class UnreachableModel(LogModel):
 	def append_cmd_unreachable(cls, options):
 		cls._append_cmd(cls.cmd_unreachable, options)
 
-	def get_lines(self):
+	def get_lines(self) -> 'LogLines':
 		self.cmd = self.cmd_unreachable
 		try:
 			lines = Model.get_lines(self)
@@ -663,7 +669,7 @@ class UnreachableModel(LogModel):
 			self.cmd_log = self._copy_command(self.cmd_log[-1])
 		else:
 			self.cmd_log = self._copy_command(self.cmd_log)
-		self.append_cmd_log.__func__(self, ["--no-walk"])
+		self.append_cmd_log.__func__(self, ["--no-walk"])  # type: ignore [attr-defined]  # I am using this class method as an instance method in order to add '--no-walk' to this instance without changing the template in the class attribute
 		self.cmd_log += lines
 		log = LogModel.get_lines(self)
 		self.cmd_log = original_cmd_log
@@ -958,7 +964,7 @@ class DetailsModel(Model):
 
 		return [([(settings.COLOR_DETAILS_UNTRACKED, utils.colored_str_to_markup(formatter.format(pattern, filename=fn), self.app.define_color))], TYPE_UNTRACKED, FileInfo(path(fn))) for fn in filenames]
 
-	def format_command_error(self, e: CommandError, action: 'str|None' = None, setting: 'str|None' = None):
+	def format_command_error(self, e: CommandError, action: 'str|None' = None, setting: 'str|None' = None) -> 'LinesOfUrwidMarkup':
 		cmd = "cmd = %r\n" % (e.cmd,)
 		err = e.err
 		if action:
@@ -1178,7 +1184,7 @@ class DetailsModel(Model):
 			'title.todo.details-view',
 		],
 	)
-	def todo(self, args: 'collections.abc.Sequence[str]') -> 'list[str|list[object]|tuple[str, object]]':
+	def todo(self, args: 'collections.abc.Sequence[str]') -> 'LinesOfUrwidMarkup':
 		if args:
 			cmd = ['$todo'] + list(args)
 			raise CommandError(cmd, "todo takes no arguments")
@@ -1248,7 +1254,7 @@ class DetailsModel(Model):
 		description = _('Show a list of commits referencing the current commit'),
 		settings = ['details-view.cmd.commit.referenced-by', 'details-view.commit.line-pattern-referenced-by'],
 	)
-	def referencedby(self, args: 'collections.abc.Sequence[str]') -> 'list[str|list[object]|tuple[str, object]]':
+	def referencedby(self, args: 'collections.abc.Sequence[str]') -> 'LinesOfUrwidMarkup':
 		assert len(args) == 1
 		hash_id = args[0]
 
@@ -1278,7 +1284,7 @@ class DetailsModel(Model):
 		description = _('Return untracked files'),
 		settings = ['details-view.show-untracked-relative-to'],
 	)
-	def untracked(self, args: 'collections.abc.Sequence[str]') -> 'list[str|list[object]|tuple[str, object]]':
+	def untracked(self, args: 'collections.abc.Sequence[str]') -> 'LinesOfUrwidMarkup':
 		assert len(args) == 0
 
 		self.cmd = ["git", "status", "--porcelain=v1"]
@@ -1318,11 +1324,11 @@ class DetailsModel(Model):
 			settings.COLOR_DECORATION_TAG,
 		],
 	)
-	def commitheader(self, args: 'collections.abc.Sequence[str]') -> 'list[str|list[object]|tuple[str, object]]':
+	def commitheader(self, args: 'collections.abc.Sequence[str]') -> 'LinesOfUrwidMarkup':
 		assert len(args) == 1
 		hash_id = args[0]
 
-		out: 'list[str|list[object]|tuple[str, object]]' = []
+		out: 'LinesOfUrwidMarkup' = []
 		values = self.get_commit_header_data(hash_id)
 		if values is None:
 			return []
