@@ -37,6 +37,7 @@ LOG_FOCUS_FIRST_LINE = "first-line"
 LOG_FOCUS_UNTRACKED = "untracked-files"
 LOG_FOCUS_UNSTAGED = "unstaged-changes"
 LOG_FOCUS_STAGED = "staged-changes"
+LOG_FOCUS_TODO = "todo-flags"
 LOG_FOCUS_LATEST_COMMIT = "latest-commit"
 
 COLOR_TITLE  = "color.title"
@@ -158,14 +159,22 @@ settings = {
 		(BOOL_TRUE,  True,  _("Show oldest stash on top.")),
 		(BOOL_FALSE, False, _("Show newest stash on top.")),
 	), _("This setting is ignored if %log-view.show-stashes% is disabled. You may want to change this setting if you change %log-view.show-stashes-as-group%.")),
-	"log-view.default-focus" : ("app.LogView.default_focus_lines", (list, (LOG_FOCUS_UNTRACKED, LOG_FOCUS_UNSTAGED, LOG_FOCUS_STAGED, LOG_FOCUS_LATEST_COMMIT, LOG_FOCUS_FIRST_LINE)), _("This setting determines which line to select when the program starts. If the first element of this list is not contained in the log view try the next.")),
-	"log-view.fmt.log" : ("app.LogModel.fmt_log", str, "The value which is used for --custom-format if the command does not contain -g/--walk-reflogs (in %log-view.cmd.log% and %details-view.cmd.commit.referenced-by%). Regarding the allowed wild cards see `git log --help` section PRETTY FORMATS."),
-	"log-view.fmt.reflog" : ("app.LogModel.fmt_reflog", str, "The value which is used for --custom-format if the command contains -g/--walk-reflogs (in %log-view.cmd.log%). Regarding the allowed wild cards see `git log --help` section PRETTY FORMATS."),
+	"log-view.show-todo-flags" : ("app.LogModel.show_list_of_todo_flags", (
+		(BOOL_TRUE,  True,  _("Always")),
+		(BOOL_FALSE, False, _("Never")),
+		(VAL_AUTO,   None,  _("If %log-view.cmd.todo.grep% has exit code 0. In case the command times out %log-view.show-todo-flags-on-timeout% decides.")),
+	), _("Whether the $todo function inserts an entry for the list of todo flags in the log view")),
+	"log-view.show-todo-flags-on-timeout" : ("app.LogModel.show_list_of_todo_flags_on_timeout", bool, _("Whether the $todo function inserts an entry for the list of todo flags in the log view if %log-view.show-todo-flags% is {auto} and %log-view.cmd.todo.grep% timed out").format(auto=VAL_AUTO)),
+	"log-view.default-focus" : ("app.LogView.default_focus_lines", (list, (LOG_FOCUS_UNTRACKED, LOG_FOCUS_UNSTAGED, LOG_FOCUS_STAGED, LOG_FOCUS_TODO, LOG_FOCUS_LATEST_COMMIT, LOG_FOCUS_FIRST_LINE)), _("This setting determines which line to select when the program starts. If the first element of this list is not contained in the log view try the next.")),
+	"log-view.fmt.log" : ("app.LogModel.fmt_log", str, "The value which is used for --custom-format if the command does not contain -g/--walk-reflogs (in %log-view.cmd.log% and %details-view.cmd.commit.referenced-by%). Regarding the allowed wildcards see `git log --help` section PRETTY FORMATS."),
+	"log-view.fmt.reflog" : ("app.LogModel.fmt_reflog", str, "The value which is used for --custom-format if the command contains -g/--walk-reflogs (in %log-view.cmd.log%). Regarding the allowed wildcards see `git log --help` section PRETTY FORMATS."),
 	"log-view.cmd.log" : ("app.LogModel.cmd_log", TYPE_COMMAND, "The command to generate the main part of the log view which shows the commits, --custom-format is replaced by %log-view.fmt.log% or %log-view.fmt.reflog%"),
 	"log-view.cmd.stash-list" : ("app.LogModel.cmd_stash_list", TYPE_COMMAND, "The command to generate the list of stashes"),
 	"log-view.cmd.unreachable" : ("app.LogModel.cmd_unreachable", TYPE_COMMAND, "The command to gather the unreachable commands if the command line argument --unreachable is used"),
+	"log-view.cmd.todo.grep" : ("app.LogModel.cmd_grep_todo_quiet", TYPE_COMMAND, _("The command used by the function $todo to check if there are todo flags if %log-view.show-todo-flags% is {auto}. A return code of 0 means there are todo flags, a return code of 1 means there are no todo flags. A different return code is an error. May contain the wildcard {wc}.").format(auto=VAL_AUTO, wc=WC_GIT_ROOT)),
 	"log-view.line-pattern" : ("app.LogModel.ln_pattern", TYPE_COLORED_STR, _("How to format special log view entries such as staged and unstaged changes. If you change %log-view.cmd.log% you may want to adapt this. The value of this setting should include the wildcard {logentry}. Additionally to the color markup which can be used this is colored with one of %color.log-entry.*%.")),
-	"log-view.line-pattern-error" : ("app.LogModel.ln_pattern_error", TYPE_COLORED_STR, _("This is displayed if one of the git commands %log-view.cmd.*% fails, the wild cards {err} and {cmd} are supported")),
+	"log-view.line-pattern-error" : ("app.LogModel.ln_pattern_error", TYPE_COLORED_STR, _("This is displayed if one of the git commands %log-view.cmd.*% fails, the wildcards {err}, {cmd} and {hint} are supported. {hint} is a hint how to avoid this error formatted with %log-view.line-pattern-error.hint% if available or an empty string otherwise.")),
+	"log-view.line-pattern-error.hint" : ("app.LogModel.ln_pattern_error_hint", TYPE_COLORED_STR, _("How to format the wildcard {hint} in %log-view.line-pattern-error% if available. Should contain the wildcard {hint}.")),
 
 	"details-view.auto-open" : ("app.is_auto_open_enabled", bool, "Automatically open the details when selecting a line. It may be useful to disable this if you are on a slow computer or are dealing with big commits."),
 	"details-view.indent-broken-code" : ("app.DetailsModel.indent_broken_code", bool, "Move the align character to the next non-whitespace character (this requires align=left-indentation and can be influenced by %app.tab%)"),
@@ -173,27 +182,27 @@ settings = {
 	"details-view.max-lines-per-file.blob" : ("app.DetailsView.max_lines_per_file_blob", int, "Trim long blobs to reduce loading time."),
 	"details-view.show-untracked-relative-to" : ("app.DetailsModel.untracked_relative", relative_values, _("How to show untracked files, see also %details-view.show-todo-relative-to%")),
 	"details-view.show-todo-relative-to" : ("app.DetailsModel.todo_relative", relative_values_todo, _("How to show file names returned by the $todo function, see also %details-view.show-untracked-relative-to%")),
-	"details-view.cmd.commit" : ("app.DetailsModel.cmd_pattern", TYPE_COMMAND, _("The command which generates the details view for a commit, should include the wild card {wc_hash}").format(wc_hash=WC_HASH_ID)),
-	"details-view.cmd.tag" : ("app.DetailsModel.cmd_pattern_tag", TYPE_COMMAND, _("The command which generates the details view for a tag, should include the wild card {wc_hash}").format(wc_hash=WC_HASH_ID)),
-	"details-view.cmd.blob" : ("app.DetailsModel.cmd_pattern_blob", TYPE_COMMAND, _("The command which generates the details view for a blob, should include the wild card {wc_hash}").format(wc_hash=WC_HASH_ID)),
-	"details-view.cmd.tree" : ("app.DetailsModel.cmd_pattern_tree", TYPE_COMMAND, _("The command which generates the details view for a tree, should include the wild card {wc_hash}").format(wc_hash=WC_HASH_ID)),
-	"details-view.cmd.stash" : ("app.DetailsModel.cmd_pattern_stashed", TYPE_COMMAND, _("The command which generates the details view for a stash, should include the wild card {wc_hash}").format(wc_hash=WC_HASH_ID)),
+	"details-view.cmd.commit" : ("app.DetailsModel.cmd_pattern", TYPE_COMMAND, _("The command which generates the details view for a commit, should include the wildcard {wc_hash}").format(wc_hash=WC_HASH_ID)),
+	"details-view.cmd.tag" : ("app.DetailsModel.cmd_pattern_tag", TYPE_COMMAND, _("The command which generates the details view for a tag, should include the wildcard {wc_hash}").format(wc_hash=WC_HASH_ID)),
+	"details-view.cmd.blob" : ("app.DetailsModel.cmd_pattern_blob", TYPE_COMMAND, _("The command which generates the details view for a blob, should include the wildcard {wc_hash}").format(wc_hash=WC_HASH_ID)),
+	"details-view.cmd.tree" : ("app.DetailsModel.cmd_pattern_tree", TYPE_COMMAND, _("The command which generates the details view for a tree, should include the wildcard {wc_hash}").format(wc_hash=WC_HASH_ID)),
+	"details-view.cmd.stash" : ("app.DetailsModel.cmd_pattern_stashed", TYPE_COMMAND, _("The command which generates the details view for a stash, should include the wildcard {wc_hash}").format(wc_hash=WC_HASH_ID)),
 	"details-view.cmd.staged" : ("app.DetailsModel.cmd_staged", TYPE_COMMAND, _("The command which generates the details view for staged changes")),
 	"details-view.cmd.unstaged" : ("app.DetailsModel.cmd_unstaged", TYPE_COMMAND, _("The command which generates the details view for unstaged changes")),
 	"details-view.cmd.untracked" : ("app.DetailsModel.cmd_untracked", TYPE_COMMAND, _("The command which generates the details view for untracked changes, each line is formatted with %details-view.unstaged.line-pattern-untracked% or %details-view.untracked.line-pattern-untracked% and colored with %color.details-view.line-pattern-untracked%")),
-	"details-view.cmd.todo" : ("app.DetailsModel.cmd_todo", TYPE_COMMAND, _("The command which generates the details view for TODO flags")),
-	"details-view.cmd.todo.grep" : ("app.DetailsModel.cmd_grep_todo", TYPE_COMMAND, _("The command used by the function $todo to generate the list of TODO flags, each line represents one TODO flag and has the pattern <filename>:<linenumber>:<linecontent>. The lines will be formatted with %details-view.todo.line-pattern%. May contain the wildcard {wc}.").format(wc=WC_GIT_ROOT)),
+	"details-view.cmd.todo" : ("app.DetailsModel.cmd_todo", TYPE_COMMAND, _("The command which generates the details view for todo flags")),
+	"details-view.cmd.todo.grep" : ("app.DetailsModel.cmd_grep_todo", TYPE_COMMAND, _("The command used by the function $todo to generate the list of todo flags, each line represents one todo flag and has the pattern <filename>:<linenumber>:<linecontent>. The lines will be formatted with %details-view.todo.line-pattern%. May contain the wildcard {wc}.").format(wc=WC_GIT_ROOT)),
 	"details-view.cmd.diff" : ("app.DiffModel.cmd", TYPE_COMMAND, _("The beginning of the command which generates the content for gitd")),
 	"details-view.cmd.commit.header-data" : ("app.DetailsModel.cmd_start_header_data", TYPE_COMMAND, _("The beginning of the command which gathers the data for the function $commitheader, format and commit hash are appended automatically")),
 	"details-view.cmd.commit.referenced-by" : ("app.DetailsModel.cmd_referencing_commits", TYPE_COMMAND, _("The beginning of the command which generates the list of commits referencing the current commit in the function $referencedby, --grep is added automatically, --custom-format is replaced by %log-view.fmt.log%, the output is formatted by %details-view.commit.line-pattern-referenced-by%")),
 	"details-view.untracked.line-pattern-untracked" : ("app.DetailsModel.ln_pattern_untracked_in_separate_group", TYPE_COLORED_STR, _("How to format untracked files displayed in a separate group (if %log-view.show-untracked-as-separate-group% is enabled or there are no other unstaged changes), should include the wildcard {filename}, additionally to the color markup which can be used this is colored with %color.details-view.line-pattern-untracked%")),
 	"details-view.unstaged.line-pattern-untracked" : ("app.DetailsModel.ln_pattern_untracked_in_unstaged_changes", TYPE_COLORED_STR, _("How to format untracked files displayed together with unstaged changes (if %log-view.show-untracked-as-separate-group% is disabled), should include the wildcard {filename}, additionally to the color markup which can be used this is colored with %color.details-view.line-pattern-untracked%")),
 	"details-view.commit.line-pattern-referenced-by" : ("app.DetailsModel.ln_pattern_referencedby", str, _("How to format an entry in the list of commits which reference the current commit, should include the wildcard {logentry}, this is related to %log-view.line-pattern% but does not support color markup, this should not contain an %app.align-character% if %details-view.cmd.commit.referenced-by% contains one")),
-	"details-view.commit.user-pattern" : ("app.DetailsModel.pattern_user", TYPE_COLORED_STR, _("How to display a user in $commitheader, supported are the wild cards {name} and {email}")),
+	"details-view.commit.user-pattern" : ("app.DetailsModel.pattern_user", TYPE_COLORED_STR, _("How to display a user in $commitheader, supported are the wildcards {name} and {email}")),
 	"details-view.commit.indentation-commit-message" : ("app.DetailsModel.indent_body", str, _("An indentation for the commit message in $commitheader")),
-	"details-view.todo.line-pattern" : ("app.DetailsModel.pattern_todo", TYPE_COLORED_STR, _("How the $todo function formats TODO flags, may include the wildcards {filename}, {linenumber} and {todoflag}")),
+	"details-view.todo.line-pattern" : ("app.DetailsModel.pattern_todo", TYPE_COLORED_STR, _("How the $todo function formats todo flags, may include the wildcards {filename}, {linenumber} and {todoflag}")),
 	"details-view.todo.strip-indentation" : ("app.DetailsModel.todo_strip_indentation", bool, _("Strip the indentation from the lines displayed by the $todo function")),
-	"details-view.todo.todo-only" : ("app.DetailsModel.todo_only", bool, _("Strip everything before the TODO from the lines displayed by the $todo function")),
+	"details-view.todo.todo-only" : ("app.DetailsModel.todo_only", bool, _("Strip everything before the todo from the lines displayed by the $todo function")),
 	"details-view.line-numbers.min-width" : ("app.DetailsModel.min_line_number_width", int, _("The amount of space to reserve for line numbers is determined for each hunk individually and depends on the biggest line number, this setting can be used to always reserve space for at least a certain number of digits so that the line numbers column has the same width for all hunks")),
 	"details-view.line-numbers.sep" : ("app.DetailsModel.linenumber_suffix", TYPE_COLORED_STR, _("A separator between the line number and the line itself")),
 	"details-view.line-numbers.sep-left-right" : ("app.DetailsModel.linenumber_sep", TYPE_COLORED_STR, _("A separator between the left and right line number (when showing the line numbers before a merge commit)")),
@@ -234,10 +243,11 @@ settings = {
 	"title.untracked-and-unstaged" : ("app.model.title_unstaged_and_untracked", str, _("Log entry and details view title for a combination of untracked files and unstaged changes (if %log-view.show-untracked-as-separate-group% is false and %log-view.show-untracked% is true)")),
 	"title.staged" : ("app.model.title_staged", str, _("Log entry and details view title for staged changes")),
 	"title.stashes" : ("app.model.title_stashes_group", str, _("Log entry and details view title for the list of all stashes (if %log-view.show-stashes% and %log-view.show-stashes-as-group% are not false)")),
-	"title.todo" : ("app.model.title_todo", str, _("Log entry and details view title for the list of TODO flags, may contain the wildcard {number}")),
+	"title.todo.log-view" : ("app.model.title_todo_log", str, _("Log entry for the list of todo flags, see also %title.todo.details-view%")),
+	"title.todo.details-view" : ("app.model.title_todo_details", str, _("Details view title for the list of todo flags, may contain the wildcard {number}, see also %title.todo.log-view%")),
 	# this is used in details-view only but seperating it from the other titles would not make sense
-	"title.commit" : ("app.model.title_commit", str, _("Title displayed in the details view for commits, the following wild cards are supported: {hash_id}, {refnames} (see %title.commit.refnames%), {subject}, {committer_name}, {committer_email}, {committer_date}, {author_name}, {author_email}, {author_date}, {body}")),
-	"title.commit.refnames" : ("app.model.pattern_refnames", str, _("A pattern how to format {refnames} in %title.commit% if the commit has ref names (decorations), this pattern should include the wild card {refnames}")),
+	"title.commit" : ("app.model.title_commit", str, _("Title displayed in the details view for commits, the following wildcards are supported: {hash_id}, {refnames} (see %title.commit.refnames%), {subject}, {committer_name}, {committer_email}, {committer_date}, {author_name}, {author_email}, {author_date}, {body}")),
+	"title.commit.refnames" : ("app.model.pattern_refnames", str, _("A pattern how to format {refnames} in %title.commit% if the commit has ref names (decorations), this pattern should include the wildcard {refnames}")),
 	"title.commit.refnames.sep" : ("app.model.refnames_sep", str, _("The seperator used in %title.commit.refnames% between different branches and tags")),
 	"title.commit.refnames.head-branch-sep" : ("app.model.DetailsModel.decoration_head_branch_sep", str, _("The seperator used in %title.commit.refnames% between HEAD and the branch which HEAD is pointing to")),
 
@@ -250,6 +260,9 @@ settings = {
 	), _("In the help of a command with arguments (when opened with `link` or `help --shortcuts`)")),
 
 	"default-editor" : ("app.model.opener.default_editor", str, _("The editor to be used if EDITOR is not set")),
+
+	"log-view.timeout" : ("app.model.LogModel.timeout", float, _("Time in seconds. If a command which generates the content of the log view takes longer than this it will be killed. A negative number disables the timeout, i.e. the command will not be killed no matter how long it takes.")),
+	"details-view.timeout" : ("app.model.DetailsModel.timeout", float, _("Time in seconds. If a command which generates the content of the details view takes longer than this it will be killed. A negative number disables the timeout, i.e. the command will not be killed no matter how long it takes.")),
 }
 
 for color in COLORS:
